@@ -29,7 +29,10 @@
 
 #ifndef _WIN32
 #include <boost/algorithm/string.hpp>
-#if !defined(ANDROID) && !defined(__ANDROID__) // CodePlex 269
+#if !defined(ANDROID) && !defined(__ANDROID__) && !defined(__GLIBC__) // CodePlex 269
+/* Systems using glibc: xlocale.h has been removed from glibc 2.26
+   The above include of locale.h is sufficient
+   Further details: https://sourceware.org/git/?p=glibc.git;a=commit;h=f0be25b6336db7492e47d2e8e72eb8af53b5506d */
 #include <xlocale.h>
 #endif
 #endif
@@ -100,28 +103,43 @@ namespace conversions
     /// </summary>
     /// <param name="s">A single byte character UTF-8 string.</param>
     /// <returns>A platform dependent string type.</returns>
+#ifdef _UTF16_STRINGS
     _ASYNCRTIMP utility::string_t __cdecl to_string_t(std::string &&s);
+#else
+    inline utility::string_t&& to_string_t(std::string &&s) { return std::move(s); }
+#endif
 
     /// <summary>
     /// Converts to a platform dependent Unicode string type.
     /// </summary>
     /// <param name="s">A two byte character UTF-16 string.</param>
     /// <returns>A platform dependent string type.</returns>
+#ifdef _UTF16_STRINGS
+    inline utility::string_t&& to_string_t(utf16string &&s) { return std::move(s); }
+#else
     _ASYNCRTIMP utility::string_t __cdecl to_string_t(utf16string &&s);
-
+#endif
     /// <summary>
     /// Converts to a platform dependent Unicode string type.
     /// </summary>
     /// <param name="s">A single byte character UTF-8 string.</param>
     /// <returns>A platform dependent string type.</returns>
+#ifdef _UTF16_STRINGS
     _ASYNCRTIMP utility::string_t __cdecl to_string_t(const std::string &s);
+#else
+    inline const utility::string_t& to_string_t(const std::string &s) { return s; }
+#endif
 
     /// <summary>
     /// Converts to a platform dependent Unicode string type.
     /// </summary>
     /// <param name="s">A two byte character UTF-16 string.</param>
     /// <returns>A platform dependent string type.</returns>
+#ifdef _UTF16_STRINGS
+    inline const utility::string_t& to_string_t(const utf16string &s) { return s; }
+#else
     _ASYNCRTIMP utility::string_t __cdecl to_string_t(const utf16string &s);
+#endif
 
     /// <summary>
     /// Converts to a UTF-16 from string.
@@ -135,14 +153,33 @@ namespace conversions
     /// </summary>
     /// <param name="value">A two byte character UTF-16 string.</param>
     /// <returns>A two byte character UTF-16 string.</returns>
-    _ASYNCRTIMP utf16string __cdecl to_utf16string(utf16string value);
+    inline const utf16string& to_utf16string(const utf16string& value)
+    {
+        return value;
+    }
+    /// <summary>
+    /// Converts to a UTF-16 from string.
+    /// </summary>
+    /// <param name="value">A two byte character UTF-16 string.</param>
+    /// <returns>A two byte character UTF-16 string.</returns>
+    inline utf16string&& to_utf16string(utf16string&& value)
+    {
+        return std::move(value);
+    }
 
     /// <summary>
     /// Converts to a UTF-8 string.
     /// </summary>
     /// <param name="value">A single byte character UTF-8 string.</param>
     /// <returns>A single byte character UTF-8 string.</returns>
-    _ASYNCRTIMP std::string __cdecl to_utf8string(std::string value);
+    inline std::string&& to_utf8string(std::string&& value) { return std::move(value); }
+
+    /// <summary>
+    /// Converts to a UTF-8 string.
+    /// </summary>
+    /// <param name="value">A single byte character UTF-8 string.</param>
+    /// <returns>A single byte character UTF-8 string.</returns>
+    inline const std::string& to_utf8string(const std::string& value) { return value; }
 
     /// <summary>
     /// Converts to a UTF-8 string.
@@ -167,7 +204,8 @@ namespace conversions
     _ASYNCRTIMP std::vector<unsigned char> __cdecl from_base64(const utility::string_t& str);
 
     template <typename Source>
-    utility::string_t print_string(const Source &val, const std::locale &loc)
+    CASABLANCA_DEPRECATED("All locale-sensitive APIs will be removed in a future update. Use stringstreams directly if locale support is required.")
+    utility::string_t print_string(const Source &val, const std::locale& loc = std::locale())
     {
         utility::ostringstream_t oss;
         oss.imbue(loc);
@@ -179,19 +217,81 @@ namespace conversions
         return oss.str();
     }
 
-    template <typename Source>
-    utility::string_t print_string(const Source &val)
-    {
-        return print_string(val, std::locale());
-    }
-
+    CASABLANCA_DEPRECATED("All locale-sensitive APIs will be removed in a future update. Use stringstreams directly if locale support is required.")
     inline utility::string_t print_string(const utility::string_t &val)
     {
         return val;
     }
 
+    namespace details
+    {
+
+#if defined(__ANDROID__)
+        template<class T>
+        inline std::string to_string(const T& t)
+        {
+            std::ostringstream os;
+            os.imbue(std::locale::classic());
+            os << t;
+            return os.str();
+        }
+#endif
+
+        template<class T>
+        inline utility::string_t to_string_t(T&& t)
+        {
+#ifdef _UTF16_STRINGS
+            using std::to_wstring;
+            return to_wstring(std::forward<T>(t));
+#else
+#if !defined(__ANDROID__)
+            using std::to_string;
+#endif
+            return to_string(std::forward<T>(t));
+#endif
+        }
+
+        template <typename Source>
+        utility::string_t print_string(const Source &val)
+        {
+            utility::ostringstream_t oss;
+            oss.imbue(std::locale::classic());
+            oss << val;
+            if (oss.bad())
+            {
+                throw std::bad_cast();
+            }
+            return oss.str();
+        }
+
+        inline const utility::string_t& print_string(const utility::string_t &val)
+        {
+            return val;
+        }
+
+        template <typename Target>
+        Target scan_string(const utility::string_t &str)
+        {
+            Target t;
+            utility::istringstream_t iss(str);
+            iss.imbue(std::locale::classic());
+            iss >> t;
+            if (iss.bad())
+            {
+                throw std::bad_cast();
+            }
+            return t;
+        }
+
+        inline const utility::string_t& scan_string(const utility::string_t &str)
+        {
+            return str;
+        }
+    }
+
     template <typename Target>
-    Target scan_string(const utility::string_t &str, const std::locale &loc)
+    CASABLANCA_DEPRECATED("All locale-sensitive APIs will be removed in a future update. Use stringstreams directly if locale support is required.")
+    Target scan_string(const utility::string_t &str, const std::locale &loc = std::locale())
     {
         Target t;
         utility::istringstream_t iss(str);
@@ -204,12 +304,7 @@ namespace conversions
         return t;
     }
 
-    template <typename Target>
-    Target scan_string(const utility::string_t &str)
-    {
-        return scan_string<Target>(str, std::locale());
-    }
-
+    CASABLANCA_DEPRECATED("All locale-sensitive APIs will be removed in a future update. Use stringstreams directly if locale support is required.")
     inline utility::string_t scan_string(const utility::string_t &str)
     {
         return str;
@@ -488,7 +583,7 @@ private:
 
 #ifdef _WIN32
     // void* to avoid pulling in windows.h
-    static _ASYNCRTIMP bool __cdecl datetime::system_type_to_datetime(/*SYSTEMTIME*/ void* psysTime, uint64_t seconds, datetime * pdt);
+    static _ASYNCRTIMP bool __cdecl system_type_to_datetime(/*SYSTEMTIME*/ void* psysTime, uint64_t seconds, datetime * pdt);
 #else
     static datetime timeval_to_datetime(const timeval &time);
 #endif
