@@ -143,13 +143,11 @@ namespace web { namespace details
         /// - The percent character ('%'), and thus any percent-endcoded octet
         /// - The sub-delimiters
         /// - ':' (colon)
-        ///
-        /// Note that we don't currently support:
-        /// - IPv6 addresses (requires '[]')
+        /// - IPv6 requires '[]' allowed for it to be valid URI and passed to underlying platform for IPv6 support
         /// </summary>
         inline bool is_authority_character(int c)
         {
-            return is_unreserved(c) || is_sub_delim(c) || c == '%' || c == '@' || c == ':';
+            return is_unreserved(c) || is_sub_delim(c) || c == '%' || c == '@' || c == ':' || c == '[' || c == ']';
         }
 
         /// <summary>
