@@ -226,7 +226,7 @@ namespace web
         uri_builder &append_query(const utility::string_t &name, const T &value, bool do_encoding = true)
         {
             auto encodedName = name;
-            auto encodedValue = ::utility::conversions::print_string(value, std::locale::classic());
+            auto encodedValue = utility::conversions::details::print_string(value);
 
             if (do_encoding)
             {
@@ -261,13 +261,13 @@ namespace web
         /// Combine and validate the URI components into a encoded string. An exception will be thrown if the URI is invalid.
         /// </summary>
         /// <returns>The created URI as a string.</returns>
-        _ASYNCRTIMP utility::string_t to_string();
+        _ASYNCRTIMP utility::string_t to_string() const;
 
         /// <summary>
         /// Combine and validate the URI components into a URI class instance. An exception will be thrown if the URI is invalid.
         /// </summary>
         /// <returns>The create URI as a URI class instance.</returns>
-        _ASYNCRTIMP uri to_uri();
+        _ASYNCRTIMP uri to_uri() const;
 
         /// <summary>
         /// Validate the generated URI from all existing components of this uri_builder.
