@@ -11,6 +11,7 @@
 * =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ****/
 #include "stdafx.h"
+#include "../common/internal_http_helpers.h"
 
 using namespace web;
 using namespace utility;
@@ -208,7 +209,7 @@ utility::size64_t http_headers::content_length() const
 
 void http_headers::set_content_length(utility::size64_t length)
 {
-    m_headers[http::header_names::content_length] = utility::conversions::print_string(length, std::locale::classic());
+    m_headers[http::header_names::content_length] = utility::conversions::details::to_string_t(length);
 }
 
 namespace details {
